package cp.gui;


import cp.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

public class PlayerListTableModel extends AbstractTableModel {
  Vector<String> columnNames = new Vector<String>();
  Players players;
  
  PlayerListTableModel() {
    initColumnNames();
  }
  
  void setPlayers(Players players) {
    this.players = players;
    update();
  }
  
  public int getRowCount() {
    if (players != null) {
      return players.size();
    }
    else {
      return 2;
    }
  }

  public int getColumnCount() {
    return columnNames.size();
  }

  public String getColumnName(int col) {
    return columnNames.get(col);
  }

  public boolean isCellEditable(int row, int col) {
    return false;
  }

  public Object getValueAt(int rowIndex, int columnIndex) {
    if (players != null) {
			Player player = players.get(rowIndex);
      if (getColumnName(columnIndex).equalsIgnoreCase("Nr")) {
        return rowIndex+1;
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Namn")) {
        return (player.toString());
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Grp")) {
        return player.getGroup();
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Klubb")) {
        return player.getClub();
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Rating")) {
        return (player.getRatingN() > 0 ? player.getRatingN() : "");
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("ELO")) {
        return (player.getRatingI() > 0 ? player.getRatingI() : "");
      }
    }
    return null;
  }
  
  public Player getPlayer(int row) {
    if (players != null && row < players.size()) {
      return players.get(row);
    }
    return null;
  }
  
  public void update() {
    if (players != null) {
      // Sort players according to rating
      Collections.sort(players);
    }
    initColumnNames();
    fireTableDataChanged();
  }

  void initColumnNames() {
    columnNames.clear();
    columnNames.add("Nr");
    columnNames.add("Namn");
    columnNames.add("Grp");
    columnNames.add("Klubb");
    columnNames.add("Rating");
    columnNames.add("ELO");
    this.fireTableStructureChanged();
  }
  
  void setColumnWidths(JTable jTable) {
    TableColumn column = null;
    for (int i = 0; i < getColumnCount(); i++) {
      column = jTable.getColumnModel().getColumn(i);
      if (jTable.getColumnName(i).equalsIgnoreCase("Nr")) {
        column.setPreferredWidth(30); 
      }
      else if (jTable.getColumnName(i).equalsIgnoreCase("Namn")) {
        column.setPreferredWidth(200);
      }
      else if (jTable.getColumnName(i).equalsIgnoreCase("Grp")) {
        column.setPreferredWidth(40);
      }
      else if (jTable.getColumnName(i).equalsIgnoreCase("Klubb")) {
        column.setPreferredWidth(100);
      }
      else if (jTable.getColumnName(i).equalsIgnoreCase("Rating")) {
        column.setPreferredWidth(45);
      }
      else if (jTable.getColumnName(i).equalsIgnoreCase("ELO")) {
        column.setPreferredWidth(45);
      }
    }
  }
  
}
