package cp.gui.web;

import cp.Round;
import cp.Tournament;

import cp.gui.EditResultsPanel;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.swing.JOptionPane;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.*;

public class HTMLPairings {

  public HTMLPairings() {}
  
  public void publish(EditResultsPanel editResultsPanel) throws FileNotFoundException, IOException {
    editResultsPanel.update();
    // Create output html file
		Round round = editResultsPanel.getRound();
		Tournament tournament = round.getTournament();
    String fileName = tournament.getResultsPage();
    if (fileName != null) {
      try {
        // Create document
        Document document = createDocument();
        Element html = document.createElement("html");
        document.appendChild(html);
        // Create title
        Element head = document.createElement("head");
        html.appendChild(head);
        Element title = document.createElement("title");
        head.appendChild(title);
        Text text = document.createTextNode("Lottning/resultat " + tournament);
        title.appendChild(text);
				appendStylesheet(document, head);
        // Create body
        Element body = document.createElement("body");
        html.appendChild(body);
        text = document.createTextNode("Lotta - Lottning/resultat fr turnering " + tournament + ", rond " + round.getRoundNr());
        body.appendChild(text);
        
        // Save the table
        editResultsPanel.save(document, body);
        
        // Save the file
        save(document, fileName);
      }
      catch (ParserConfigurationException e) {
        JOptionPane.showMessageDialog(null, "ParserConfigurationException:" + e.getMessage(), "Can't create file", JOptionPane.ERROR_MESSAGE);
      }
      catch (TransformerConfigurationException e) {
        JOptionPane.showMessageDialog(null, "Resultatfil kan inte ppnas eller skapas. Meddelande:" + e.getMessage(), "Kan inte ppna fil", JOptionPane.ERROR_MESSAGE);
      }
      catch (TransformerException e) {
        JOptionPane.showMessageDialog(null, "Resultatfil kan inte ppnas eller skapas. Meddelande:" + e.getMessage(), "Kan inte ppna fil", JOptionPane.ERROR_MESSAGE);
      }
    }
    else {
      JOptionPane.showMessageDialog(null, "Du mste ange ett namn fr html-filen i turneringsdialogen fr att kunna publicera den.", "Ingen html-fil specificerad", JOptionPane.INFORMATION_MESSAGE);
    }
  }
  
  protected Document createDocument() throws ParserConfigurationException {
    DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
    DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
    Document document = documentBuilder.newDocument();
    return document;
  }
  
  protected void save(Document document, String fileName) throws TransformerConfigurationException, TransformerException {
    TransformerFactory transformerFactory = TransformerFactory.newInstance();
    Transformer transformer = transformerFactory.newTransformer();
    transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
    transformer.setOutputProperty(OutputKeys.DOCTYPE_PUBLIC, "DTD/xhtml1-transitional.dtd");
    transformer.setOutputProperty(OutputKeys.INDENT, "yes");

    DOMSource source = new DOMSource(document);
    StreamResult result =  new StreamResult(new File(fileName));
    transformer.transform(source, result);
  }

	protected void appendStylesheet(Document document, Element head) throws FileNotFoundException, IOException {
	  // Create stylesheet
	  Element style = document.createElement("style");
	  style.setAttribute("type", "text/css");
	  style.setTextContent(HTMLCSS.getCSSText());
	  head.appendChild(style);
	  /*        Element link = document.createElement("link");
	  head.appendChild(link);
	  link.setAttribute("rel", "stylesheet");
	  link.setAttribute("type", "text/css");
	  link.setAttribute("href", "chesspairer.css");*/
	}
}
