package cp.pairing;


import cp.ChessPairerException;
import cp.Game;
import cp.Games;
import cp.ScoreLotnrComparator;
import cp.Player;
import cp.Players;
import cp.Round;
import cp.Rounds;
import cp.Tournament;

import java.util.Collections;

public class Monrad extends PairingSystem {
  
  public Monrad(Tournament tournament) {
    super(tournament);
    setNewLots();
  }

  private void setNewLots() {
    Rounds rounds = tournament.getRounds();
    // First set the current lot numbers
    if (rounds.size() == 0) {
      initialize();
    }
    else {
      // Set the lot number for each player as they were in the last round
      players.setLotNr(rounds.get(rounds.size() - 1));
    }
    // Then sort the players
    Round round = null;
    if (rounds.size() > 0) {
      round = rounds.lastElement();
    }
    Collections.sort(players, new ScoreLotnrComparator(round));
    // Set the new lot numbers
    for (int i = 0; i < players.size(); i++) {
      Player player = players.get(i);
      player.setLotNr(i + 1);
    }
  }

  protected Games pair(Players players) throws ChessPairerException {
    // First check if barred pairing is possible
    if (tournament.isBarredPairing()) {
      if (players.isOneClubInMajority()) {
        return null;
      }
    }
    Games games = new Games(tournament);
    Games forbiddenGames = new Games(tournament);
    // For each player:
    for (int i=0; i < players.size(); i++) {
      Player a = players.get(i);
      // for each remaining player:
      boolean canMeetSomeone = false;
      for (int j=i+1; j < players.size(); j++) {
        Player b = players.get(j);
        // if the opponent does not meet anyone and they has not met each other
        if (!a.hasMet(b) && !isBarred(a,b)) {
          // if pair is not in forbidden list
          if (!hasMet(a, b, forbiddenGames)) {
            canMeetSomeone = true;
            Game game;
            // create game and lot the rest
            if (b.getWhiteCount() > a.getWhiteCount()) {
              game = new Game(games, a, b);
            }
            else {
              game = new Game(games, b, a);
            }
            int apos = players.indexOf(a);
            int bpos = players.indexOf(b);
            players.remove(a);
            players.remove(b);
            Games rest = pair(players);
            if (apos < bpos) {
              players.insertElementAt(a, apos);
              players.insertElementAt(b, bpos);
            }
            else {
              players.insertElementAt(b, bpos);
              players.insertElementAt(a, apos);
            }
            Round round = null;
            if (tournament.getRounds().size() > 0) {
              round = tournament.getRounds().lastElement();
            }
            if (rest == null && players.size() > 2) {
              // Can not pair. Move game to forbidden games
              forbiddenGames.add(game);
              // player = highest player in the moved pair
              i = players.indexOf(game.getHighestPlayer()) - 1;
            }
            else {
              games.add(game);
              if (rest != null) {
                games.addAll(rest);
              }
              return games;
            }
          }
        }
      }
      if (!canMeetSomeone) {
        return null;
      }
    }
    if (games.isEmpty()) {
      return null;
    }
    return games;
  }

}
